# Template C++ Project

[![Language Server: Clangd](https://img.shields.io/badge/Language%20Server-Clangd-blue.svg)](https://clangd.llvm.org/)
[![Code Formatter: ClangFormat](https://img.shields.io/badge/Code%20Formatter-ClangFormat-blue.svg)](https://clang.llvm.org/docs/ClangFormat.html)

**WARNING** : this setup only works for Linux and Windows-WSL.

## Setup and Run

In order to setup your C++ project and run it, you must use the following commands :

Create the build output directory (the first time)

```bash
mkdir build
cd build
```

Compile your code

```bash
cmake .. # Re-run everytime you add or remove .hpp & .cpp files, or when you change CMakeLIsts.txt
make # Compile your code
./<executable_name> # Run your compiled program
```

## VSCode Project Settings

In order to ensure that everyone uses the same linting and formatting settings,
we duplicate the relevant settings in [`.vscode/settings.json`](.vscode/settings.json).

Relevant settings with c++ :

- format on save
- using `clangd` language server over the standard one
- using `clang-format` c++ formatter

## CMake & Make

C++ projects can be configured using `cmake` and `make`.

CMake enables configuring the build environment. Its configuration is found in the [`CMakeLists.txt`](./CMakeLists.txt) file.
Notably, running the `cmake` command generate a `Makefile`, which is used by `make`.

Make is a build automation tool. It is used to compile your code. Its configuration is found in the [`Makefile`](./Makefile) file, which is autogenerated by `cmake`. It is a shortcut to compile your C++ code.

## Formatter - ClangFormat

[ClangFormat](https://clang.llvm.org/docs/ClangFormat.html) is a good c++ formatter.
Using a formatter ensures that the code you and your team write is consistent.
You should always use a formatter.

You can edit the formatter style using the `clang-format.style` setting in [`settings.json`](.vscode/settings.json).

## Language Server - Clangd

The [Clangd](https://clangd.llvm.org/) language server is a better language server than the default one used by VSCode C++ extensions. It also provides inlay hints. In order for it to work, you must install the Clangd extension, and disable the default C++ extension's intellisense.

## Environment and reproducibility

As c++ does not have a package manager, you must install dependencies by hand. There are 3 ways to install dependencies :

- Install using Linux packages : some dependencies are available as Linux packages.
- Install manually to `/usr/local/include` (headers), `/usr/local/lib` (libraries) and `/usr/local/bin` (executables). If you put the relevant dependency folders and executable there, your C++ compiler will find them without any issue.
- Install manually to a random location : you will need to modify your `CMakeLists.txt` in order to tell the compiler where to find the dependencies (more complicated).

### Git

Some common gitignores for a C++ project :

- `build/` : the build output folder.
- `.cache/` : clangd cache folder.

## C++ Clean Code

Some often-overlooked C++ clean code rules :

### Docstrings

Always write docstrings in order to describe the purpose of your functions and classes.

```cpp
/** This function does something */
void function() {
}
```

### Header files

When your project gets bigger, you must split your code into header (`.hpp`) and source (`.cpp`) files.
The header files contain class and function header declarations, and source files contain the implementations.

- ALWAYS add `#pragma once` at the very top of each of your header files.
- ALWAYS include header files in your code, NEVER include source files.
