# Template Typescript Project

[![Node.js Version](https://img.shields.io/badge/Node.js-20-brightgreen.svg)](https://nodejs.org/)
[![ESLint](https://img.shields.io/badge/Linter-ESLint-blue.svg)](https://eslint.org/)
[![Formatter-Prettier](https://img.shields.io/badge/Formatter-Prettier-yellow.svg)](https://prettier.io/)

## Node environment

Javascript code can run in 2 environments :

- Inside a web browser
- Using Node js

When developping websites or backend applications, we use Node js. You can download Node js on their [official website](https://nodejs.org/en).

### Package Manager

Node js dependencies are managed using a package manager (npm, yarn, pnpm...). The default package manager is `npm`.
It comes bundled with Node js.

Node dependencies are installed in the `node_modules/` folder.

Every project has a `package.json` file, which contains the project's **dependencies**, **metadata** and **scripts** shorthands.

You can use the following commands :

```bash
npm install # Install all dependencies listed in package.json
npm install <package> # Install a package and add it to package.json
npm install <package> --save-dev # Install a package as a dev dependency

# Scripts we defined in package.json
npm run build # Build the project into javascript files. The output directory is build/
npm run execute # Execute the project. The entry point is build/index.js
npm run format # Run the prettier formatter on all files (this is done automatically upon saving each file)
npm run lint # Run the eslint linter on all files (this is done automatically by VSCode when opening or changing a file)
```

### Why use (or not) typescript ?

TypeScript is a strongly typed superset of JavaScript. It is recommended to ALWAYS use TypeScript for medium to large projects, especially if multiple developers will be working on it.

However, for small html-css websites, using small JavaScript files may be the better option.

### How to initialize projects

This repo contains an example setup for a node js application. However, if you want to create a website or a node js application using specific frameworks, you may instead initialize your project using one of the following commands :

#### Websites

```bash
npm create vite@latest my-react-website -- --template react-ts # Create a vanilla React app using Typescript
npx create-next-app@latest # Create a Next js website. Be sure to use Typescript and ESLint in the given options !
npm create svelte@latest my-app # Create a Svelte app. Be sure to use Typescript, ESLint and Prettier in the given options !
```

#### Node js applications

```bash
npm i -g @nestjs/cli && nest new my-backend-app # Create a Nest js backend application
npm init adonis-ts-app@latest my-backend-app # Create an Adonis js backend application
```

#### Final recommendations

- Be sure to set `"strict": true` in your `tsconfig.json` file, else using TypeScript will not be as useful.
- The frameworks presented in the last 2 sections are recommended (and popular enough).

## VSCode Project Settings

Relevant settings with TypeScript :

- format on save
- order imports on save
- update imports when moving files around

## Formatter - Prettier

[Prettier](https://prettier.io/) is a popular javascript formatter which is strict and opinionated. It also handles html, css and json files (the whole web dev stack).
Using a formatter ensures that the code you and your team write is consistent.
You should always use a formatter.

The formatter is configured in the [`.prettierrc.js`](./.prettierrc.js) file.

## Linter - ESLint

[ESLint](https://eslint.org/) is a popular javascript linter, which is strict and configurable.
Using a linter will help you write cleaner code, and avoid common mistakes.
You should always use a linter.

The linter is configured in the [`.eslintrc.js`](./.eslintrc.js) file.

### Git

Some common gitignores for a TypeScript project :

- `node_modules/` : the `npm` dependencies.
- `build/` : the output build.
